==> 3.1.4.tar.gz: ios/Classes/ActivityRecognitionManager.swift <==
//
//  ActivityRecognitionManager.swift
//  activity_rec_flutter
//
//  Created by WOO JIN HWANG on 2021/04/23.
//

import CoreMotion
import Foundation
import SQLite3

typealias ActivityDataHandler = (ActivityData) -> Void

class ActivityRecognitionManager {

  private let motionActivityManager = CMMotionActivityManager()
  private var isRunningService = false

  // Create an instance of UserDefaultsManager
  let userDefaultsManager = UserDefaultsManager.shared

  let bikeCO2 = 255
  let runningCO2 = 265
  let walkCO2 = 271

  let perMinBikeDistance = 0.246
  let perMinBicycleDistance = 0.150
  let perMinRunningDistance = 0.140
  let perMinWalkDistance = 0.07435

  public func startService(handler: @escaping ActivityDataHandler) {
    if (isRunningService) {
      NSLog("ActivityRecognitionManager The activity recognition service has already started.")
      stopService()
    }

    motionActivityManager.startActivityUpdates(to: .main) { (activity) in
      guard let activity = activity else { return }

      let actData = ActivityData(from: activity)
      //let currentTime = String(Date().timeIntervalSince1970 * 1000)

      if(actData.type != "UNKNOWN"){
      print("ActivityRecognitionManager \(actData.type) activity tracked ...")
      //todo write point calculation logic here
            let recordedActivity = self.userDefaultsManager.recordedActivity
            let recordedActivityTime = self.userDefaultsManager.recordedActivityTime

             print("ActivityRecognitionManager activity type: \(actData.type), confidence: \(actData.confidence), startDate: \(actData.startDate)")
             print("ActivityRecognitionManager userDefaultsManager recordedActivity =====> \(recordedActivity)")
             print("ActivityRecognitionManager userDefaultsManager recordedActivityTime =====> \(recordedActivityTime)")

             if(recordedActivity == ""){
              print("ActivityRecognitionManager =====> recordedActivity == null")
              self.userDefaultsManager.recordedActivity = actData.type
              self.userDefaultsManager.recordedActivityTime = actData.startDate

             }else{
             print("ActivityRecognitionManager =====> recordedActivity != null")
              if (actData.type != recordedActivity) {
                 print("ActivityRecognitionManager =====> activityData != recordedActivity")

                  self.userDefaultsManager.recordedActivity = actData.type
                  self.userDefaultsManager.recordedActivityTime = Date()
                  let timeIntervalInSeconds = Date().timeIntervalSince(recordedActivityTime)
                  let timeIntervalInMinutes = Int(timeIntervalInSeconds / 60)
                  print("ActivityRecognitionManager mini difference is \(timeIntervalInSeconds) seconds.")
                  print("ActivityRecognitionManager mini difference is \(timeIntervalInMinutes) minutes.")

                  let activityType = self.userDefaultsManager.recordedActivity
                  print("ActivityRecognitionManager mini activityType \(activityType)")
                     switch activityType {
                     case "IN_VEHICLE":
                            let distance = Int(Double(timeIntervalInMinutes) * perMinBikeDistance)
                            let co2Vehicle = Int(Double(distance) * bikeCO2 / 1000.0)
                            let points = Int(1.2 * Double(distance) * Double(co2Vehicle))

                            //let points = Int(Double(timeIntervalInMinutes) * 0.29) // per minute points for vehicle
                            let records =  DatabaseManager.shared.getActivityPoints()
                            let numberOfRecords = records.count
                            print("ActivityRecognitionManager IN_VEHICLE records retrieved: \(numberOfRecords)")
                            // Check if there are any records returned.
                            if let firstRecord = records.first {
                                let vehiclePointsOfFirstRecord = firstRecord.vehiclePoints
                                print("ActivityRecognitionManager IN_VEHICLE points: \(vehiclePointsOfFirstRecord)")
                                // edit record logic
                                DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                             vehiclePoints: firstRecord.vehiclePoints + points,
                                                                             bicyclePoints: firstRecord.bicyclePoints,
                                                                             runningPoints: firstRecord.runningPoints,
                                                                             walkingPoints: firstRecord.walkingPoints,
                                                                             stillPoints: firstRecord.stillPoints)

                            } else {
                                print("ActivityRecognitionManager IN_VEHICLE No records retrieved.")
                                // add record logic
                                DatabaseManager.shared.insertActivityPoints(vehiclePoints:points,
                                bicyclePoints:0,
                                runningPoints:0,
                                walkingPoints:0,
                                stillPoints:0)
                            }

                     case "ON_BICYCLE":
                         let distance = Int(Double(timeIntervalInMinutes) * perMinBicycleDistance)
                         let co2Bicycle = Int(Double(distance) * walkCO2 / 1000.0)
                         let points = Int(1.2 * Double(distance) * Double(co2Bicycle))

                         //let points = Int(Double(timeIntervalInMinutes) * 0.50) // per minute points for bicycle
                         let records =  DatabaseManager.shared.getActivityPoints()
                         let numberOfRecords = records.count
                         print("ActivityRecognitionManager ON_BICYCLE records retrieved: \(numberOfRecords)")
                         // Check if there are any records returned.
                         if let firstRecord = records.first {
                             let vehiclePointsOfFirstRecord = firstRecord.bicyclePoints
                             print("ActivityRecognitionManager ON_BICYCLE points: \(vehiclePointsOfFirstRecord)")
                             // edit record logic
                             DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                         vehiclePoints: firstRecord.vehiclePoints,
                                                                         bicyclePoints: firstRecord.bicyclePoints + points,
                                                                         runningPoints: firstRecord.runningPoints,
                                                                         walkingPoints: firstRecord.walkingPoints,
                                                                         stillPoints: firstRecord.stillPoints)
                         } else {
                             print("ActivityRecognitionManager ON_BICYCLE No records retrieved.")
                             // add record logic
                             DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                       bicyclePoints:points,
                                                       runningPoints:0,
                                                       walkingPoints:0,
                                                       stillPoints:0)
                         }

                     case "RUNNING":
                         let distance = Int(Double(timeIntervalInMinutes) * perMinRunningDistance)
                         let co2Running = Int(Double(distance) * runningCO2 / 1000.0)
                         let points = Int(1.5 * Double(distance) * Double(co2Running))

                         //let points = Int(Double(timeIntervalInMinutes) * 0.45) // per minute points for running
                         let records =  DatabaseManager.shared.getActivityPoints()
                         let numberOfRecords = records.count
                         print("ActivityRecognitionManager RUNNING records retrieved: \(numberOfRecords)")
                         // Check if there are any records returned.
                         if let firstRecord = records.first {
                             let vehiclePointsOfFirstRecord = firstRecord.runningPoints
                             print("ActivityRecognitionManager RUNNING points: \(vehiclePointsOfFirstRecord)")
                             // edit record logic
                             DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                        vehiclePoints: firstRecord.vehiclePoints,
                                                                        bicyclePoints: firstRecord.bicyclePoints,
                                                                        runningPoints: firstRecord.runningPoints + points,
                                                                        walkingPoints: firstRecord.walkingPoints,
                                                                        stillPoints: firstRecord.stillPoints)
                         } else {
                             print("ActivityRecognitionManager RUNNING No records retrieved.")
                             // add record logic
                             DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                                              bicyclePoints:0,
                                                                              runningPoints:points,
                                                                              walkingPoints:0,
                                                                              stillPoints:0)
                         }

                     case "WALKING":
                         let distance = Int(Double(timeIntervalInMinutes) * perMinBikeDistance)
                         let co2Walking = Int(Double(distance) * walkCO2 / 1000.0)
                         let points = Int(2.3 * Double(distance) * Double(co2Walking))

                         //let points = Int(Double(timeIntervalInMinutes) * 0.3) // per minute points for walking
                         let records =  DatabaseManager.shared.getActivityPoints()
                         let numberOfRecords = records.count
                         print("ActivityRecognitionManager WALKING records retrieved: \(numberOfRecords)")
                         // Check if there are any records returned.
                         if let firstRecord = records.first {
                             let vehiclePointsOfFirstRecord = firstRecord.walkingPoints
                             print("ActivityRecognitionManager WALKING points: \(vehiclePointsOfFirstRecord)")
                             // edit record logic
                             DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                        vehiclePoints: firstRecord.vehiclePoints,
                                                                        bicyclePoints: firstRecord.bicyclePoints,
                                                                        runningPoints: firstRecord.runningPoints,
                                                                        walkingPoints: firstRecord.walkingPoints + points,
                                                                        stillPoints: firstRecord.stillPoints)
                         } else {
                             print("ActivityRecognitionManager WALKING No records retrieved.")
                             // add record logic
                             DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                                        bicyclePoints:0,
                                                                        runningPoints:0,
                                                                        walkingPoints:points,
                                                                        stillPoints:0)
                         }

                     case "STILL":
                         let points = Int(Double(timeIntervalInMinutes) * 0.1) // per minute points for still
                         let records =  DatabaseManager.shared.getActivityPoints()
                         let numberOfRecords = records.count
                         print("ActivityRecognitionManager STILL records retrieved: \(numberOfRecords)")
                         if let firstRecord = records.first {
                             let vehiclePointsOfFirstRecord = firstRecord.stillPoints
                             print("ActivityRecognitionManager STILL points: \(vehiclePointsOfFirstRecord)")
                             // edit record logic
                             DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                        vehiclePoints: firstRecord.vehiclePoints,
                                                                        bicyclePoints: firstRecord.bicyclePoints,
                                                                        runningPoints: firstRecord.runningPoints,
                                                                        walkingPoints: firstRecord.walkingPoints,
                                                                        stillPoints: firstRecord.stillPoints + points)
                         } else {
                             print("ActivityRecognitionManager STILL No records retrieved.")
                             // add record logic
                             DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                                        bicyclePoints:0,
                                                                        runningPoints:0,
                                                                        walkingPoints:0,
                                                                        stillPoints:points)
                         }


                     default:
                         print("UNKNOWN activity")
                     }


              }
             }
        DatabaseManager.shared.insertActivityData(type: actData.type, confidence: actData.confidence,startDate:actData.startDate)
      }else{
      print("ActivityRecognitionManager UNKNOWN activity tracked ...")
      }
      handler(ActivityData(from: activity))
    }
    isRunningService = true
  }

  public func stopService() {
    motionActivityManager.stopActivityUpdates()
    isRunningService = false
  }
}

==> 3.1.7.tar.gz: ios/Classes/ActivityRecognitionManager.swift <==
//
//  ActivityRecognitionManager.swift
//  activity_rec_flutter
//
//  Created by WOO JIN HWANG on 2021/04/23.
//

import CoreMotion
import Foundation
import SQLite3

typealias ActivityDataHandler = (ActivityData) -> Void

class ActivityRecognitionManager {

  private let motionActivityManager = CMMotionActivityManager()
  private var isRunningService = false

  // Create an instance of UserDefaultsManager
  let userDefaultsManager = UserDefaultsManager.shared

  let bikeCO2 = 255.0
  let runningCO2 = 265.0
  let walkCO2 = 271.0

  let perMinBikeDistance = 0.246
  let perMinBicycleDistance = 0.150
  let perMinRunningDistance = 0.140
  let perMinWalkDistance = 0.07435

  public func startService(handler: @escaping ActivityDataHandler) {
    if (isRunningService) {
      NSLog("ActivityRecognitionManager The activity recognition service has already started.")
      stopService()
    }

    motionActivityManager.startActivityUpdates(to: .main) { (activity) in
      guard let activity = activity else { return }

      let actData = ActivityData(from: activity)
      //let currentTime = String(Date().timeIntervalSince1970 * 1000)

      if(actData.type != "UNKNOWN"){
      print("ActivityRecognitionManager \(actData.type) activity tracked ...")
      //todo write point calculation logic here
            let recordedActivity = self.userDefaultsManager.recordedActivity
            let recordedActivityTime = self.userDefaultsManager.recordedActivityTime

             print("ActivityRecognitionManager activity type: \(actData.type), confidence: \(actData.confidence), startDate: \(actData.startDate)")
             print("ActivityRecognitionManager userDefaultsManager recordedActivity =====> \(recordedActivity)")
             print("ActivityRecognitionManager userDefaultsManager recordedActivityTime =====> \(recordedActivityTime)")

             if(recordedActivity == ""){
              print("ActivityRecognitionManager =====> recordedActivity == null")
              self.userDefaultsManager.recordedActivity = actData.type
              self.userDefaultsManager.recordedActivityTime = actData.startDate

             }else{
             print("ActivityRecognitionManager =====> recordedActivity != null")
              if (actData.type != recordedActivity) {
                 print("ActivityRecognitionManager =====> activityData != recordedActivity")

                  self.userDefaultsManager.recordedActivity = actData.type
                  self.userDefaultsManager.recordedActivityTime = Date()
                  let timeIntervalInSeconds = Date().timeIntervalSince(recordedActivityTime)
                  let timeIntervalInMinutes = Int(timeIntervalInSeconds / 60)
                  print("ActivityRecognitionManager mini difference is \(timeIntervalInSeconds) seconds.")
                  print("ActivityRecognitionManager mini difference is \(timeIntervalInMinutes) minutes.")

                  let activityType = self.userDefaultsManager.recordedActivity
                  print("ActivityRecognitionManager mini activityType \(activityType)")
                     switch activityType {
                     case "IN_VEHICLE":
                            let distance = Int(Double(timeIntervalInMinutes) * self.perMinBikeDistance)
                            let co2Vehicle = Int(Double(distance) * self.bikeCO2 / 1000.0)
                            let points = Int(1.2 * Double(distance) * Double(co2Vehicle))

                            //let points = Int(Double(timeIntervalInMinutes) * 0.29) // per minute points for vehicle
                            let records =  DatabaseManager.shared.getActivityPoints()
                            let numberOfRecords = records.count
                            print("ActivityRecognitionManager IN_VEHICLE records retrieved: \(numberOfRecords)")
                            // Check if there are any records returned.
                            if let firstRecord = records.first {
                                let vehiclePointsOfFirstRecord = firstRecord.vehiclePoints
                                print("ActivityRecognitionManager IN_VEHICLE points: \(vehiclePointsOfFirstRecord)")
                                // edit record logic
                                DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                             vehiclePoints: firstRecord.vehiclePoints + points,
                                                                             bicyclePoints: firstRecord.bicyclePoints,
                                                                             runningPoints: firstRecord.runningPoints,
                                                                             walkingPoints: firstRecord.walkingPoints,
                                                                             stillPoints: firstRecord.stillPoints)

                            } else {
                                print("ActivityRecognitionManager IN_VEHICLE No records retrieved.")
                                // add record logic
                                DatabaseManager.shared.insertActivityPoints(vehiclePoints:points,
                                bicyclePoints:0,
                                runningPoints:0,
                                walkingPoints:0,
                                stillPoints:0)
                            }

                     case "ON_BICYCLE":
                         let distance = Int(Double(timeIntervalInMinutes) * self.perMinBicycleDistance)
                         let co2Bicycle = Int(Double(distance) * self.walkCO2 / 1000.0)
                         let points = Int(1.2 * Double(distance) * Double(co2Bicycle))

                         //let points = Int(Double(timeIntervalInMinutes) * 0.50) // per minute points for bicycle
                         let records =  DatabaseManager.shared.getActivityPoints()
                         let numberOfRecords = records.count
                         print("ActivityRecognitionManager ON_BICYCLE records retrieved: \(numberOfRecords)")
                         // Check if there are any records returned.
                         if let firstRecord = records.first {
                             let vehiclePointsOfFirstRecord = firstRecord.bicyclePoints
                             print("ActivityRecognitionManager ON_BICYCLE points: \(vehiclePointsOfFirstRecord)")
                             // edit record logic
                             DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                         vehiclePoints: firstRecord.vehiclePoints,
                                                                         bicyclePoints: firstRecord.bicyclePoints + points,
                                                                         runningPoints: firstRecord.runningPoints,
                                                                         walkingPoints: firstRecord.walkingPoints,
                                                                         stillPoints: firstRecord.stillPoints)
                         } else {
                             print("ActivityRecognitionManager ON_BICYCLE No records retrieved.")
                             // add record logic
                             DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                       bicyclePoints:points,
                                                       runningPoints:0,
                                                       walkingPoints:0,
                                                       stillPoints:0)
                         }

                     case "RUNNING":
                         let distance = Int(Double(timeIntervalInMinutes) * self.perMinRunningDistance)
                         let co2Running = Int(Double(distance) * self.runningCO2 / 1000.0)
                         let points = Int(1.5 * Double(distance) * Double(co2Running))

                         //let points = Int(Double(timeIntervalInMinutes) * 0.45) // per minute points for running
                         let records =  DatabaseManager.shared.getActivityPoints()
                         let numberOfRecords = records.count
                         print("ActivityRecognitionManager RUNNING records retrieved: \(numberOfRecords)")
                         // Check if there are any records returned.
                         if let firstRecord = records.first {
                             let vehiclePointsOfFirstRecord = firstRecord.runningPoints
                             print("ActivityRecognitionManager RUNNING points: \(vehiclePointsOfFirstRecord)")
                             // edit record logic
                             DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                        vehiclePoints: firstRecord.vehiclePoints,
                                                                        bicyclePoints: firstRecord.bicyclePoints,
                                                                        runningPoints: firstRecord.runningPoints + points,
                                                                        walkingPoints: firstRecord.walkingPoints,
                                                                        stillPoints: firstRecord.stillPoints)
                         } else {
                             print("ActivityRecognitionManager RUNNING No records retrieved.")
                             // add record logic
                             DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                                              bicyclePoints:0,
                                                                              runningPoints:points,
                                                                              walkingPoints:0,
                                                                              stillPoints:0)
                         }

                     case "WALKING":
                         let distance = Int(Double(timeIntervalInMinutes) * self.perMinBikeDistance)
                         let co2Walking = Int(Double(distance) * self.walkCO2 / 1000.0)
                         let points = Int(2.3 * Double(distance) * Double(co2Walking))

                         //let points = Int(Double(timeIntervalInMinutes) * 0.3) // per minute points for walking
                         let records =  DatabaseManager.shared.getActivityPoints()
                         let numberOfRecords = records.count
                         print("ActivityRecognitionManager WALKING records retrieved: \(numberOfRecords)")
                         // Check if there are any records returned.
                         if let firstRecord = records.first {
                             let vehiclePointsOfFirstRecord = firstRecord.walkingPoints
                             print("ActivityRecognitionManager WALKING points: \(vehiclePointsOfFirstRecord)")
                             // edit record logic
                             DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                        vehiclePoints: firstRecord.vehiclePoints,
                                                                        bicyclePoints: firstRecord.bicyclePoints,
                                                                        runningPoints: firstRecord.runningPoints,
                                                                        walkingPoints: firstRecord.walkingPoints + points,
                                                                        stillPoints: firstRecord.stillPoints)
                         } else {
                             print("ActivityRecognitionManager WALKING No records retrieved.")
                             // add record logic
                             DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                                        bicyclePoints:0,
                                                                        runningPoints:0,
                                                                        walkingPoints:points,
                                                                        stillPoints:0)
                         }

                     case "STILL":
                         let points = Int(Double(timeIntervalInMinutes) * 0.1) // per minute points for still
                         let records =  DatabaseManager.shared.getActivityPoints()
                         let numberOfRecords = records.count
                         print("ActivityRecognitionManager STILL records retrieved: \(numberOfRecords)")
                         if let firstRecord = records.first {
                             let vehiclePointsOfFirstRecord = firstRecord.stillPoints
                             print("ActivityRecognitionManager STILL points: \(vehiclePointsOfFirstRecord)")
                             // edit record logic
                             DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                        vehiclePoints: firstRecord.vehiclePoints,
                                                                        bicyclePoints: firstRecord.bicyclePoints,
                                                                        runningPoints: firstRecord.runningPoints,
                                                                        walkingPoints: firstRecord.walkingPoints,
                                                                        stillPoints: firstRecord.stillPoints + points)
                         } else {
                             print("ActivityRecognitionManager STILL No records retrieved.")
                             // add record logic
                             DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                                        bicyclePoints:0,
                                                                        runningPoints:0,
                                                                        walkingPoints:0,
                                                                        stillPoints:points)
                         }


                     default:
                         print("UNKNOWN activity")
                     }


              }
             }
        DatabaseManager.shared.insertActivityData(type: actData.type, confidence: actData.confidence,startDate:actData.startDate)
      }else{
      print("ActivityRecognitionManager UNKNOWN activity tracked ...")
      }
      handler(ActivityData(from: activity))
    }
    isRunningService = true
  }

  public func stopService() {
    motionActivityManager.stopActivityUpdates()
    isRunningService = false
  }
}
